@@ -2,6 +2,7 @@ package main
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"io"
 	"log"
@@ -15,8 +16,8 @@ import (
 	"github.com/dustin/go-humanize"
 	"github.com/spf13/cobra"
 
-	"git.sr.ht/~emersion/hut/srht/gitsrht"
-	"git.sr.ht/~emersion/hut/termfmt"
+	"git.sr.ht/~xenrox/hut/srht/gitsrht"
+	"git.sr.ht/~xenrox/hut/termfmt"
 )
 
 func newGitCommand() *cobra.Command {
@@ -28,12 +29,13 @@ func newGitCommand() *cobra.Command {
 	cmd.AddCommand(newGitCreateCommand())
 	cmd.AddCommand(newGitListCommand())
 	cmd.AddCommand(newGitDeleteCommand())
+	cmd.AddCommand(newGitCloneCommand())
 	cmd.AddCommand(newGitACLCommand())
 	cmd.AddCommand(newGitShowCommand())
 	cmd.AddCommand(newGitUserWebhookCommand())
 	cmd.AddCommand(newGitUpdateCommand())
 	cmd.PersistentFlags().StringP("repo", "r", "", "name of repository")
-	cmd.RegisterFlagCompletionFunc("repo", completeRepo)
+	cmd.RegisterFlagCompletionFunc("repo", completeGitRepo)
 	return cmd
 }
 
@@ -67,19 +69,20 @@ func newGitCreateCommand() *cobra.Command {
 
 		log.Printf("Created repository %s\n", repo.Name)
 
+		ver, err := gitsrht.SshSettings(c.Client, ctx)
+		if err != nil {
+			log.Fatalf("failed to retrieve settings: %v", err)
+		}
+
+		u, err := url.Parse(c.BaseURL)
+		if err != nil {
+			log.Fatalf("failed to parse base URL: %v", err)
+		}
+
+		cloneURL := fmt.Sprintf("%s@%s:%s/%s", ver.Settings.SshUser, u.Hostname(),
+			repo.Owner.CanonicalName, repo.Name)
+
 		if clone {
-			ver, err := gitsrht.SshSettings(c.Client, ctx)
-			if err != nil {
-				log.Fatalf("failed to retrieve settings: %v", err)
-			}
-
-			u, err := url.Parse(c.BaseURL)
-			if err != nil {
-				log.Fatalf("failed to parse base URL: %v", err)
-			}
-
-			cloneURL := fmt.Sprintf("%s@%s:%s/%s", ver.Settings.SshUser, u.Hostname(),
-				repo.Owner.CanonicalName, repo.Name)
 			cloneCmd := exec.Command("git", "clone", cloneURL)
 			cloneCmd.Stdin = os.Stdin
 			cloneCmd.Stdout = os.Stdout
@@ -89,6 +92,8 @@ func newGitCreateCommand() *cobra.Command {
 			if err != nil {
 				log.Fatal(err)
 			}
+		} else {
+			fmt.Printf("%s\n", cloneURL)
 		}
 	}
 
@@ -117,16 +122,17 @@ func newGitListCommand() *cobra.Command {
 		if len(args) > 0 {
 			owner, instance = parseOwnerName(args[0])
 		}
-		pagerify(func(p pager) bool {
+
+		err := pagerify(func(p pager) error {
 			var repos *gitsrht.RepositoryCursor
 			if len(owner) > 0 {
 				c := createClientWithInstance("git", cmd, instance)
 				username := strings.TrimLeft(owner, ownerPrefixes)
 				user, err := gitsrht.RepositoriesByUser(c.Client, ctx, username, cursor)
 				if err != nil {
-					log.Fatal(err)
+					return err
 				} else if user == nil {
-					log.Fatal("no such user")
+					return errors.New("no such user")
 				}
 				repos = user.Repositories
 			} else {
@@ -134,7 +140,7 @@ func newGitListCommand() *cobra.Command {
 				var err error
 				repos, err = gitsrht.Repositories(c.Client, ctx, cursor)
 				if err != nil {
-					log.Fatal(err)
+					return err
 				}
 			}
 
@@ -143,8 +149,15 @@ func newGitListCommand() *cobra.Command {
 			}
 
 			cursor = repos.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -175,14 +188,14 @@ func newGitDeleteCommand() *cobra.Command {
 			name, owner, instance = parseResourceName(args[0])
 		} else {
 			var err error
-			name, owner, instance, err = getRepoName(ctx, cmd)
+			name, owner, instance, err = getGitRepoName(ctx, cmd)
 			if err != nil {
 				log.Fatal(err)
 			}
 		}
 
 		c := createClientWithInstance("git", cmd, instance)
-		id := getRepoID(c, ctx, name, owner)
+		id := getGitRepoID(c, ctx, name, owner)
 
 		if !autoConfirm && !getConfirmation(fmt.Sprintf("Do you really want to delete the repo %s", name)) {
 			log.Println("Aborted")
@@ -200,13 +213,80 @@ func newGitDeleteCommand() *cobra.Command {
 		Use:               "delete [repo]",
 		Short:             "Delete a repository",
 		Args:              cobra.MaximumNArgs(1),
-		ValidArgsFunction: completeRepo,
+		ValidArgsFunction: completeGitRepo,
 		Run:               run,
 	}
 	cmd.Flags().BoolVarP(&autoConfirm, "yes", "y", false, "auto confirm")
 	return cmd
 }
 
+func newGitCloneCommand() *cobra.Command {
+	run := func(cmd *cobra.Command, args []string) {
+		log.Println("Cloning repository")
+		cloneCmd := exec.Command("git", "clone", args[0])
+		cloneCmd.Stdin = os.Stdin
+		cloneCmd.Stdout = os.Stdout
+		cloneCmd.Stderr = os.Stderr
+
+		err := cloneCmd.Run()
+		if err != nil {
+			log.Fatalf("failed to clone repo: %v", err)
+		}
+
+		s := strings.Split(args[0], "/")
+		dir, err := os.Getwd()
+		if err != nil {
+			log.Fatalf("failed to get current directory: %v", err)
+		}
+
+		err = os.Chdir(filepath.Join(dir, s[len(s)-1]))
+		if err != nil {
+			log.Fatalf("failed to change current working directory: %v", err)
+		}
+
+		cfg, err := loadProjectConfig()
+		if err != nil {
+			log.Fatalf("failed to load project config: %v", err)
+		}
+
+		if cfg != nil {
+			if cfg.DevList != "" {
+				log.Printf("Configuring repository for %q\n", "git send-email")
+
+				sendemailCmd := exec.Command("git", "config", "sendemail.to", cfg.DevList)
+				sendemailCmd.Stdin = os.Stdin
+				sendemailCmd.Stdout = os.Stdout
+				sendemailCmd.Stderr = os.Stderr
+
+				err = sendemailCmd.Run()
+				if err != nil {
+					log.Fatalf("failed to set %q: %v", "git config sendemail.to", err)
+				}
+			}
+
+			if cfg.PatchPrefix {
+				prefixCmd := exec.Command("git", "config", "format.subjectPrefix", fmt.Sprintf("PATCH %s", s[len(s)-1]))
+				prefixCmd.Stdin = os.Stdin
+				prefixCmd.Stdout = os.Stdout
+				prefixCmd.Stderr = os.Stderr
+
+				err = prefixCmd.Run()
+				if err != nil {
+					log.Fatalf("failed to set %q: %v", "git config format.subjectPrefix", err)
+				}
+			}
+		}
+	}
+	cmd := &cobra.Command{
+		Use:               "clone <URL>",
+		Short:             "Clone a repository",
+		Args:              cobra.ExactArgs(1),
+		ValidArgsFunction: cobra.NoFileCompletions,
+		Run:               run,
+	}
+	return cmd
+}
+
 func newGitArtifactCommand() *cobra.Command {
 	cmd := &cobra.Command{
 		Use:   "artifact",
@@ -222,14 +302,14 @@ func newGitArtifactUploadCommand() *cobra.Command {
 	var rev string
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
-		repoName, owner, instance, err := getRepoName(ctx, cmd)
+		repoName, owner, instance, err := getGitRepoName(ctx, cmd)
 		if err != nil {
 			log.Fatal(err)
 		}
 
 		c := createClientWithInstance("git", cmd, instance)
 		c.HTTP.Timeout = fileTransferTimeout
-		repoID := getRepoID(c, ctx, repoName, owner)
+		repoID := getGitRepoID(c, ctx, repoName, owner)
 
 		if rev == "" {
 			var err error
@@ -272,7 +352,7 @@ func newGitArtifactListCommand() *cobra.Command {
 
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
-		repoName, owner, instance, err := getRepoName(ctx, cmd)
+		repoName, owner, instance, err := getGitRepoName(ctx, cmd)
 		if err != nil {
 			log.Fatal(err)
 		}
@@ -368,7 +448,7 @@ func newGitACLListCommand() *cobra.Command {
 			name, owner, instance = parseResourceName(args[0])
 		} else {
 			var err error
-			name, owner, instance, err = getRepoName(ctx, cmd)
+			name, owner, instance, err = getGitRepoName(ctx, cmd)
 			if err != nil {
 				log.Fatal(err)
 			}
@@ -385,7 +465,7 @@ func newGitACLListCommand() *cobra.Command {
 			username = strings.TrimLeft(owner, ownerPrefixes)
 		}
 
-		pagerify(func(p pager) bool {
+		err = pagerify(func(p pager) error {
 			if username != "" {
 				user, err = gitsrht.AclByUser(c.Client, ctx, username, name, cursor)
 			} else {
@@ -393,11 +473,11 @@ func newGitACLListCommand() *cobra.Command {
 			}
 
 			if err != nil {
-				log.Fatal(err)
+				return err
 			} else if user == nil {
-				log.Fatal("no such user")
+				return errors.New("no such user")
 			} else if user.Repository == nil {
-				log.Fatalf("no such repository %q", name)
+				return fmt.Errorf("no such repository %q", name)
 			}
 
 			for _, acl := range user.Repository.Acls.Results {
@@ -405,15 +485,22 @@ func newGitACLListCommand() *cobra.Command {
 			}
 
 			cursor = user.Repository.Acls.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
-		Use:               "list",
+		Use:               "list [repo]",
 		Short:             "List ACL entries",
 		Args:              cobra.MaximumNArgs(1),
-		ValidArgsFunction: cobra.NoFileCompletions,
+		ValidArgsFunction: completeGitRepo,
 		Run:               run,
 	}
 	return cmd
@@ -444,13 +531,13 @@ func newGitACLUpdateCommand() *cobra.Command {
 			log.Fatal("user must be in canonical form")
 		}
 
-		name, owner, instance, err := getRepoName(ctx, cmd)
+		name, owner, instance, err := getGitRepoName(ctx, cmd)
 		if err != nil {
 			log.Fatal(err)
 		}
 
 		c := createClientWithInstance("git", cmd, instance)
-		id := getRepoID(c, ctx, name, owner)
+		id := getGitRepoID(c, ctx, name, owner)
 
 		acl, err := gitsrht.UpdateACL(c.Client, ctx, id, accessMode, args[0])
 		if err != nil {
@@ -468,7 +555,7 @@ func newGitACLUpdateCommand() *cobra.Command {
 		Run:               run,
 	}
 	cmd.Flags().StringVarP(&mode, "mode", "m", "", "access mode")
-	cmd.RegisterFlagCompletionFunc("mode", completeAccessMode)
+	cmd.RegisterFlagCompletionFunc("mode", completeRepoAccessMode)
 	cmd.MarkFlagRequired("mode")
 	return cmd
 }
@@ -512,7 +599,7 @@ func newGitShowCommand() *cobra.Command {
 			name, owner, instance = parseResourceName(args[0])
 		} else {
 			var err error
-			name, owner, instance, err = getRepoName(ctx, cmd)
+			name, owner, instance, err = getGitRepoName(ctx, cmd)
 			if err != nil {
 				log.Fatal(err)
 			}
@@ -585,7 +672,7 @@ func newGitShowCommand() *cobra.Command {
 		Use:               "show [repo]",
 		Short:             "Shows a repository",
 		Args:              cobra.MaximumNArgs(1),
-		ValidArgsFunction: completeRepo,
+		ValidArgsFunction: completeGitRepo,
 		Run:               run,
 	}
 
@@ -639,7 +726,7 @@ func newGitUserWebhookCreateCommand() *cobra.Command {
 	cmd.Flags().StringSliceVarP(&events, "events", "e", nil, "webhook events")
 	cmd.RegisterFlagCompletionFunc("events", completeGitUserWebhookEvents)
 	cmd.MarkFlagRequired("events")
-	cmd.Flags().BoolVar(&stdin, "stdin", false, "read webhook query from stdin")
+	cmd.Flags().BoolVar(&stdin, "stdin", !isStdinTerminal, "read webhook query from stdin")
 	cmd.Flags().StringVarP(&url, "url", "u", "", "payload url")
 	cmd.RegisterFlagCompletionFunc("url", cobra.NoFileCompletions)
 	cmd.MarkFlagRequired("url")
@@ -652,10 +739,10 @@ func newGitUserWebhookListCommand() *cobra.Command {
 		c := createClient("git", cmd)
 		var cursor *gitsrht.Cursor
 
-		pagerify(func(p pager) bool {
+		err := pagerify(func(p pager) error {
 			webhooks, err := gitsrht.UserWebhooks(c.Client, ctx, cursor)
 			if err != nil {
-				log.Fatal(err)
+				return err
 			}
 
 			for _, webhook := range webhooks.Results {
@@ -663,8 +750,15 @@ func newGitUserWebhookListCommand() *cobra.Command {
 			}
 
 			cursor = webhooks.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -714,14 +808,14 @@ func newGitUpdateCommand() *cobra.Command {
 			name, owner, instance = parseResourceName(args[0])
 		} else {
 			var err error
-			name, owner, instance, err = getRepoName(ctx, cmd)
+			name, owner, instance, err = getGitRepoName(ctx, cmd)
 			if err != nil {
 				log.Fatal(err)
 			}
 		}
 
 		c := createClientWithInstance("git", cmd, instance)
-		id := getRepoID(c, ctx, name, owner)
+		id := getGitRepoID(c, ctx, name, owner)
 		var input gitsrht.RepoInput
 
 		if visibility != "" {
@@ -733,7 +827,14 @@ func newGitUpdateCommand() *cobra.Command {
 		}
 
 		if cmd.Flags().Changed("description") {
-			input.Description = &description
+			if description == "" {
+				_, err := gitsrht.ClearDescription(c.Client, ctx, id)
+				if err != nil {
+					log.Fatalf("failed to clear description: %v", err)
+				}
+			} else {
+				input.Description = &description
+			}
 		}
 
 		if branch != "" {
@@ -777,7 +878,7 @@ func newGitUpdateCommand() *cobra.Command {
 		Use:               "update [repo]",
 		Short:             "Update a repository",
 		Args:              cobra.MaximumNArgs(1),
-		ValidArgsFunction: completeRepo,
+		ValidArgsFunction: completeGitRepo,
 		Run:               run,
 	}
 	cmd.Flags().StringVarP(&visibility, "visibility", "v", "", "repository visibility")
@@ -790,7 +891,7 @@ func newGitUpdateCommand() *cobra.Command {
 	return cmd
 }
 
-func getRepoName(ctx context.Context, cmd *cobra.Command) (repoName, owner, instance string, err error) {
+func getGitRepoName(ctx context.Context, cmd *cobra.Command) (repoName, owner, instance string, err error) {
 	repoName, err = cmd.Flags().GetString("repo")
 	if err != nil {
 		return "", "", "", err
@@ -837,7 +938,7 @@ func guessGitRepoName(ctx context.Context, cmd *cobra.Command) (repoName, owner,
 	return "", "", "", fmt.Errorf("no sr.ht Git repository found in current directory")
 }
 
-func getRepoID(c *Client, ctx context.Context, name, owner string) int32 {
+func getGitRepoID(c *Client, ctx context.Context, name, owner string) int32 {
 	var (
 		user     *gitsrht.User
 		username string
@@ -907,12 +1008,12 @@ func guessRev() (string, error) {
 	return strings.TrimSpace(string(out)), nil
 }
 
-func completeRepo(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+func completeGitRepo(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 	ctx := cmd.Context()
 	c := createClient("git", cmd)
 	var repoList []string
 
-	repos, err := gitsrht.RepoNames(c.Client, ctx)
+	repos, err := gitsrht.CompleteRepositories(c.Client, ctx)
 	if err != nil {
 		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
@@ -947,11 +1048,9 @@ func completeRev(cmd *cobra.Command, args []string, toComplete string) ([]string
 	return revs, cobra.ShellCompDirectiveNoFileComp
 }
 
-var completeAccessMode = cobra.FixedCompletions([]string{"RO", "RW"}, cobra.ShellCompDirectiveNoFileComp)
-
 func completeArtifacts(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 	ctx := cmd.Context()
-	repoName, owner, instance, err := getRepoName(ctx, cmd)
+	repoName, owner, instance, err := getGitRepoName(ctx, cmd)
 	if err != nil {
 		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
@@ -1013,7 +1112,7 @@ func completeGitUserWebhookID(cmd *cobra.Command, args []string, toComplete stri
 
 func completeBranches(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 	ctx := cmd.Context()
-	repoName, owner, instace, err := getRepoName(ctx, cmd)
+	repoName, owner, instace, err := getGitRepoName(ctx, cmd)
 	if err != nil {
 		return nil, cobra.ShellCompDirectiveNoFileComp
 	}
