@@ -17,8 +17,8 @@ import (
 
 	"github.com/spf13/cobra"
 
-	"git.sr.ht/~emersion/hut/srht/listssrht"
-	"git.sr.ht/~emersion/hut/termfmt"
+	"git.sr.ht/~xenrox/hut/srht/listssrht"
+	"git.sr.ht/~xenrox/hut/termfmt"
 )
 
 func newListsCommand() *cobra.Command {
@@ -28,6 +28,7 @@ func newListsCommand() *cobra.Command {
 	}
 	cmd.AddCommand(newListsDeleteCommand())
 	cmd.AddCommand(newListsListCommand())
+	cmd.AddCommand(newListsUpdateCommand())
 	cmd.AddCommand(newListsSubscribeCommand())
 	cmd.AddCommand(newListsUnsubscribeCommand())
 	cmd.AddCommand(newListsCreateCommand())
@@ -101,21 +102,22 @@ func newListsListCommand() *cobra.Command {
 		if len(args) > 0 {
 			username = strings.TrimLeft(args[0], ownerPrefixes)
 		}
-		pagerify(func(p pager) bool {
+
+		err := pagerify(func(p pager) error {
 			var lists *listssrht.MailingListCursor
 			if len(username) > 0 {
 				user, err := listssrht.MailingListsByUser(c.Client, ctx, username, cursor)
 				if err != nil {
-					log.Fatal(err)
+					return err
 				} else if user == nil {
-					log.Fatal("no such user")
+					return errors.New("no such user")
 				}
 				lists = user.Lists
 			} else {
 				var err error
 				user, err := listssrht.MailingLists(c.Client, ctx, cursor)
 				if err != nil {
-					log.Fatal(err)
+					return err
 				}
 				lists = user.Lists
 			}
@@ -125,8 +127,14 @@ func newListsListCommand() *cobra.Command {
 			}
 
 			cursor = lists.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 	return cmd
 }
@@ -138,6 +146,106 @@ func printList(w io.Writer, list *listssrht.MailingList) {
 	}
 }
 
+func newListsUpdateCommand() *cobra.Command {
+	var visibility string
+	var description bool
+	run := func(cmd *cobra.Command, args []string) {
+		ctx := cmd.Context()
+		var name, owner, instance string
+		if len(args) > 0 {
+			name, owner, instance = parseMailingListName(args[0])
+		} else {
+			var err error
+			name, owner, instance, err = getMailingListName(ctx, cmd)
+			if err != nil {
+				log.Fatal(err)
+			}
+		}
+
+		c := createClientWithInstance("lists", cmd, instance)
+		id := getMailingListID(c, ctx, name, owner)
+		// TODO: Support permitMime, rejectMime
+		var input listssrht.MailingListInput
+
+		if visibility != "" {
+			listVisibility, err := listssrht.ParseVisibility(visibility)
+			if err != nil {
+				log.Fatal(err)
+			}
+			input.Visibility = &listVisibility
+		}
+
+		if description {
+			if !isStdinTerminal {
+				b, err := io.ReadAll(os.Stdin)
+				if err != nil {
+					log.Fatalf("failed to read description: %v", err)
+				}
+				description := string(b)
+				input.Description = &description
+			} else {
+				var (
+					err      error
+					user     *listssrht.User
+					username string
+				)
+
+				if owner != "" {
+					username = strings.TrimLeft(owner, ownerPrefixes)
+					user, err = listssrht.MailingListDescriptionByUser(c.Client, ctx, username, name)
+				} else {
+					user, err = listssrht.MailingListDescription(c.Client, ctx, name)
+				}
+
+				if err != nil {
+					log.Fatalf("failed to fetch description: %v", err)
+				} else if user == nil {
+					log.Fatalf("no such user %q", username)
+				} else if user.List == nil {
+					log.Fatalf("no such mailing list %q", name)
+				}
+
+				var prefill string
+				if user.List.Description != nil {
+					prefill = *user.List.Description
+				}
+
+				text, err := getInputWithEditor("hut_description*.md", prefill)
+				if err != nil {
+					log.Fatalf("failed to read description: %v", err)
+				}
+
+				if strings.TrimSpace(text) == "" {
+					_, err := listssrht.ClearDescription(c.Client, ctx, id)
+					if err != nil {
+						log.Fatalf("failed to clear description: %v", err)
+					}
+				} else {
+					input.Description = &text
+				}
+			}
+		}
+
+		_, err := listssrht.UpdateMailingList(c.Client, ctx, id, input)
+		if err != nil {
+			log.Fatal(err)
+		}
+		log.Printf("Updated mailing list\n")
+	}
+
+	cmd := &cobra.Command{
+		Use:               "update [list]",
+		Short:             "Update a mailing list",
+		Args:              cobra.MaximumNArgs(1),
+		ValidArgsFunction: completeList,
+		Run:               run,
+	}
+	cmd.Flags().StringVarP(&visibility, "visibility", "v", "", "mailing list visibility")
+	cmd.RegisterFlagCompletionFunc("visibility", completeVisibility)
+	cmd.Flags().BoolVar(&description, "description", false, "edit description")
+	return cmd
+}
+
 func newListsSubscribeCommand() *cobra.Command {
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
@@ -263,7 +371,7 @@ func newListsCreateCommand() *cobra.Command {
 		ValidArgsFunction: cobra.NoFileCompletions,
 		Run:               run,
 	}
-	cmd.Flags().BoolVar(&stdin, "stdin", false, "read mailing list from stdin")
+	cmd.Flags().BoolVar(&stdin, "stdin", !isStdinTerminal, "read mailing list from stdin")
 	cmd.Flags().StringVarP(&visibility, "visibility", "v", "public", "mailing list visibility")
 	cmd.RegisterFlagCompletionFunc("visibility", completeVisibility)
 	return cmd
@@ -286,7 +394,6 @@ func newListsArchiveCommand() *cobra.Command {
 		}
 
 		c := createClientWithInstance("lists", cmd, instance)
-		c.HTTP.Timeout = fileTransferTimeout
 
 		var (
 			user     *listssrht.User
@@ -315,6 +422,7 @@ func newListsArchiveCommand() *cobra.Command {
 			url = fmt.Sprintf("%s?since=%d", url, days)
 		}
 
+		c.HTTP.Timeout = fileTransferTimeout
 		req, err := http.NewRequestWithContext(ctx, http.MethodGet, string(url), nil)
 		if err != nil {
 			log.Fatalf("Failed to create request to fetch archive: %v", err)
@@ -390,7 +498,7 @@ func newListsPatchsetListCommand() *cobra.Command {
 			}
 		}
 
-		pagerify(func(p pager) bool {
+		err = pagerify(func(p pager) error {
 			if byUser {
 				if username != "" {
 					user, err = listssrht.PatchesByUser(c.Client, ctx, name, cursor)
@@ -399,9 +507,9 @@ func newListsPatchsetListCommand() *cobra.Command {
 				}
 
 				if err != nil {
-					log.Fatal(err)
+					return err
 				} else if user == nil {
-					log.Fatalf("no such user %q", name)
+					return fmt.Errorf("no such user %q", name)
 				}
 
 				patches = user.Patches
@@ -413,11 +521,11 @@ func newListsPatchsetListCommand() *cobra.Command {
 				}
 
 				if err != nil {
-					log.Fatal(err)
+					return err
 				} else if user == nil {
-					log.Fatalf("no such user %q", username)
+					return fmt.Errorf("no such user %q", username)
 				} else if user.List == nil {
-					log.Fatalf("no such list %q", name)
+					return fmt.Errorf("no such list %q", name)
 				}
 
 				patches = user.List.Patches
@@ -428,8 +536,15 @@ func newListsPatchsetListCommand() *cobra.Command {
 			}
 
 			cursor = patches.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -635,7 +750,7 @@ func newListsACLListCommand() *cobra.Command {
 			username = strings.TrimLeft(owner, ownerPrefixes)
 		}
 
-		pagerify(func(p pager) bool {
+		err = pagerify(func(p pager) error {
 			if username != "" {
 				user, err = listssrht.AclByUser(c.Client, ctx, username, name, cursor)
 			} else {
@@ -643,11 +758,11 @@ func newListsACLListCommand() *cobra.Command {
 			}
 
 			if err != nil {
-				log.Fatal(err)
+				return err
 			} else if user == nil {
-				log.Fatalf("no such user %q", username)
+				return fmt.Errorf("no such user %q", username)
 			} else if user.List == nil {
-				log.Fatalf("no such list %q", name)
+				return fmt.Errorf("no such list %q", name)
 			}
 
 			if cursor == nil {
@@ -665,8 +780,15 @@ func newListsACLListCommand() *cobra.Command {
 			}
 
 			cursor = user.List.Acl.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -765,7 +887,7 @@ func newListsUserWebhookCreateCommand() *cobra.Command {
 	cmd.Flags().StringSliceVarP(&events, "events", "e", nil, "webhook events")
 	cmd.RegisterFlagCompletionFunc("events", completeListsUserWebhookEvents)
 	cmd.MarkFlagRequired("events")
-	cmd.Flags().BoolVar(&stdin, "stdin", false, "read webhook query from stdin")
+	cmd.Flags().BoolVar(&stdin, "stdin", !isStdinTerminal, "read webhook query from stdin")
 	cmd.Flags().StringVarP(&url, "url", "u", "", "payload url")
 	cmd.RegisterFlagCompletionFunc("url", cobra.NoFileCompletions)
 	cmd.MarkFlagRequired("url")
@@ -778,10 +900,10 @@ func newListsUserWebhookListCommand() *cobra.Command {
 		c := createClient("lists", cmd)
 		var cursor *listssrht.Cursor
 
-		pagerify(func(p pager) bool {
+		err := pagerify(func(p pager) error {
 			webhooks, err := listssrht.UserWebhooks(c.Client, ctx, cursor)
 			if err != nil {
-				log.Fatal(err)
+				return err
 			}
 
 			for _, webhook := range webhooks.Results {
@@ -789,8 +911,15 @@ func newListsUserWebhookListCommand() *cobra.Command {
 			}
 
 			cursor = webhooks.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -890,7 +1019,7 @@ func newListsWebhookCreateCommand() *cobra.Command {
 	cmd.Flags().StringSliceVarP(&events, "events", "e", nil, "webhook events")
 	cmd.RegisterFlagCompletionFunc("events", completeMailingListWebhookEvents)
 	cmd.MarkFlagRequired("events")
-	cmd.Flags().BoolVar(&stdin, "stdin", false, "read webhook query from stdin")
+	cmd.Flags().BoolVar(&stdin, "stdin", !isStdinTerminal, "read webhook query from stdin")
 	cmd.Flags().StringVarP(&url, "url", "u", "", "payload url")
 	cmd.RegisterFlagCompletionFunc("url", cobra.NoFileCompletions)
 	cmd.MarkFlagRequired("url")
@@ -924,7 +1053,7 @@ func newListsWebhookListCommand() *cobra.Command {
 			username = strings.TrimLeft(owner, ownerPrefixes)
 		}
 
-		pagerify(func(p pager) bool {
+		err = pagerify(func(p pager) error {
 			if username != "" {
 				user, err = listssrht.MailingListWebhooksByUser(c.Client, ctx, username, name, cursor)
 			} else {
@@ -932,11 +1061,11 @@ func newListsWebhookListCommand() *cobra.Command {
 			}
 
 			if err != nil {
-				log.Fatal(err)
+				return err
 			} else if user == nil {
-				log.Fatalf("no such user %q", username)
+				return fmt.Errorf("no such user %q", username)
 			} else if user.List == nil {
-				log.Fatalf("no such mailing list %q", name)
+				return fmt.Errorf("no such mailing list %q", name)
 			}
 
 			for _, webhook := range user.List.Webhooks.Results {
@@ -944,8 +1073,15 @@ func newListsWebhookListCommand() *cobra.Command {
 			}
 
 			cursor = user.List.Webhooks.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -992,10 +1128,10 @@ func newListsSubscriptions() *cobra.Command {
 		c := createClient("lists", cmd)
 		var cursor *listssrht.Cursor
 
-		pagerify(func(p pager) bool {
+		err := pagerify(func(p pager) error {
 			subscriptions, err := listssrht.Subscriptions(c.Client, ctx, cursor)
 			if err != nil {
-				log.Fatal(err)
+				return err
 			}
 
 			for _, sub := range subscriptions.Results {
@@ -1003,8 +1139,15 @@ func newListsSubscriptions() *cobra.Command {
 			}
 
 			cursor = subscriptions.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -1076,6 +1219,15 @@ func getMailingListName(ctx context.Context, cmd *cobra.Command) (name, owner, i
 		return name, owner, instance, nil
 	}
 
+	cfg, err := loadProjectConfig()
+	if err != nil {
+		return "", "", "", err
+	}
+	if cfg != nil && cfg.DevList != "" {
+		name, owner, instance = parseMailingListName(cfg.DevList)
+		return name, owner, instance, nil
+	}
+
 	name, owner, instance, err = guessMailingListName(ctx)
 	if err != nil {
 		return "", "", "", err
