@@ -1,7 +1,6 @@
 package main
 
 import (
-	"context"
 	"errors"
 	"fmt"
 	"io"
@@ -12,10 +11,11 @@ import (
 	"strings"
 
 	"github.com/dustin/go-humanize"
+	"github.com/juju/ansiterm/tabwriter"
 	"github.com/spf13/cobra"
 
-	"git.sr.ht/~emersion/hut/srht/metasrht"
-	"git.sr.ht/~emersion/hut/termfmt"
+	"git.sr.ht/~xenrox/hut/srht/metasrht"
+	"git.sr.ht/~xenrox/hut/termfmt"
 )
 
 func newMetaCommand() *cobra.Command {
@@ -25,9 +25,11 @@ func newMetaCommand() *cobra.Command {
 	}
 	cmd.AddCommand(newMetaShowCommand())
 	cmd.AddCommand(newMetaAuditLogCommand())
+	cmd.AddCommand(newMetaUpdateCommand())
 	cmd.AddCommand(newMetaSSHKeyCommand())
 	cmd.AddCommand(newMetaPGPKeyCommand())
 	cmd.AddCommand(newMetaUserWebhookCommand())
+	cmd.AddCommand(newMetaOAuthCommand())
 	return cmd
 }
 
@@ -82,10 +84,10 @@ func newMetaAuditLogCommand() *cobra.Command {
 		c := createClient("meta", cmd)
 		var cursor *metasrht.Cursor
 
-		pagerify(func(p pager) bool {
+		err := pagerify(func(p pager) error {
 			logs, err := metasrht.AuditLog(c.Client, ctx, cursor)
 			if err != nil {
-				log.Fatal(err)
+				return err
 			}
 
 			for _, log := range logs.Results {
@@ -93,8 +95,15 @@ func newMetaAuditLogCommand() *cobra.Command {
 			}
 
 			cursor = logs.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -118,6 +127,102 @@ func printAuditLog(w io.Writer, log *metasrht.AuditLogEntry) {
 	fmt.Fprintln(w, s)
 }
 
+func newMetaUpdateCommand() *cobra.Command {
+	var email, location, url string
+	var bio bool
+	run := func(cmd *cobra.Command, args []string) {
+		ctx := cmd.Context()
+		c := createClient("meta", cmd)
+		var input metasrht.UserInput
+
+		if bio {
+			if !isStdinTerminal {
+				b, err := io.ReadAll(os.Stdin)
+				if err != nil {
+					log.Fatalf("failed to read bio: %v", err)
+				}
+				biography := string(b)
+				input.Bio = &biography
+			} else {
+				me, err := metasrht.Bio(c.Client, ctx)
+				if err != nil {
+					log.Fatalf("failed to fetch bio: %v", err)
+				}
+
+				var prefill string
+				if me.Bio != nil {
+					prefill = *me.Bio
+				}
+
+				text, err := getInputWithEditor("hut_bio*.md", prefill)
+				if err != nil {
+					log.Fatalf("failed to read bio: %v", err)
+				}
+
+				if strings.TrimSpace(text) == "" {
+					_, err := metasrht.ClearBio(c.Client, ctx)
+					if err != nil {
+						log.Fatalf("failed to clear bio: %v", err)
+					}
+				} else {
+					input.Bio = &text
+				}
+			}
+		}
+
+		if cmd.Flags().Changed("email") {
+			input.Email = &email
+		}
+
+		if cmd.Flags().Changed("location") {
+			if location == "" {
+				_, err := metasrht.ClearUserLocation(c.Client, ctx)
+				if err != nil {
+					log.Fatalf("failed to clear location: %v", err)
+				}
+			} else {
+				input.Location = &location
+			}
+		}
+
+		if cmd.Flags().Changed("url") {
+			if url == "" {
+				_, err := metasrht.ClearUserURL(c.Client, ctx)
+				if err != nil {
+					log.Fatalf("failed to clear URL: %v", err)
+				}
+			} else {
+				input.Url = &url
+			}
+		}
+
+		_, err := metasrht.UpdateUser(c.Client, ctx, &input)
+		if err != nil {
+			log.Fatal(err)
+		}
+
+		log.Println("Successfully updated account")
+		if cmd.Flags().Changed("email") {
+			log.Printf("An email has been sent to %q to confirm the change\n", email)
+		}
+	}
+	cmd := &cobra.Command{
+		Use:               "update",
+		Short:             "Update account",
+		Args:              cobra.ExactArgs(0),
+		ValidArgsFunction: cobra.NoFileCompletions,
+		Run:               run,
+	}
+	cmd.Flags().BoolVar(&bio, "bio", false, "edit biography")
+	cmd.Flags().StringVar(&email, "email", "", "email")
+	cmd.RegisterFlagCompletionFunc("email", cobra.NoFileCompletions)
+	cmd.Flags().StringVar(&location, "location", "", "location")
+	cmd.RegisterFlagCompletionFunc("location", cobra.NoFileCompletions)
+	cmd.Flags().StringVar(&url, "url", "", "URL")
+	cmd.RegisterFlagCompletionFunc("url", cobra.NoFileCompletions)
+	return cmd
+}
+
 func newMetaSSHKeyCommand() *cobra.Command {
 	cmd := &cobra.Command{
 		Use:   "ssh-key",
@@ -245,7 +350,7 @@ func newMetaSSHKeyListCommand() *cobra.Command {
 			username = strings.TrimLeft(args[0], ownerPrefixes)
 		}
 
-		pagerify(func(p pager) bool {
+		err = pagerify(func(p pager) error {
 			if username != "" {
 				if raw {
 					user, err = metasrht.ListRawSSHKeysByUser(c.Client, ctx, username, cursor)
@@ -261,9 +366,9 @@ func newMetaSSHKeyListCommand() *cobra.Command {
 			}
 
 			if err != nil {
-				log.Fatal(err)
+				return err
 			} else if user == nil {
-				log.Fatalf("no such user %q", username)
+				return fmt.Errorf("no such user %q", username)
 			}
 
 			if raw {
@@ -281,8 +386,15 @@ func newMetaSSHKeyListCommand() *cobra.Command {
 			}
 
 			cursor = user.SshKeys.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -319,7 +431,7 @@ func newMetaPGPKeyCreateCommand() *cobra.Command {
 		if len(args) > 0 {
 			keyBytes, err = os.ReadFile(args[0])
 		} else {
-			keyBytes, err = exportDefaultPGPKey(ctx)
+			keyBytes, err = exportDefaultPGPKey()
 		}
 		if err != nil {
 			log.Fatal(err)
@@ -346,7 +458,7 @@ func newMetaPGPKeyCreateCommand() *cobra.Command {
 	}
 }
 
-func exportDefaultPGPKey(ctx context.Context) ([]byte, error) {
+func exportDefaultPGPKey() ([]byte, error) {
 	out, err := exec.Command("gpg", "--list-secret-keys", "--with-colons").Output()
 	if err != nil {
 		return nil, fmt.Errorf("failed to list keys in GPG keyring: %v", err)
@@ -424,7 +536,7 @@ func newMetaPGPKeyListCommand() *cobra.Command {
 			username = strings.TrimLeft(args[0], ownerPrefixes)
 		}
 
-		pagerify(func(p pager) bool {
+		err = pagerify(func(p pager) error {
 			if username != "" {
 				if raw {
 					user, err = metasrht.ListRawPGPKeysByUser(c.Client, ctx, username, cursor)
@@ -440,9 +552,9 @@ func newMetaPGPKeyListCommand() *cobra.Command {
 			}
 
 			if err != nil {
-				log.Fatal(err)
+				return err
 			} else if user == nil {
-				log.Fatalf("no such user %q", username)
+				return fmt.Errorf("no such user %q", username)
 			}
 
 			if raw {
@@ -457,8 +569,15 @@ func newMetaPGPKeyListCommand() *cobra.Command {
 			}
 
 			cursor = user.PgpKeys.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -520,7 +639,7 @@ func newMetaUserWebhookCreateCommand() *cobra.Command {
 	cmd.Flags().StringSliceVarP(&events, "events", "e", nil, "webhook events")
 	cmd.RegisterFlagCompletionFunc("events", completeMetaUserWebhookEvents)
 	cmd.MarkFlagRequired("events")
-	cmd.Flags().BoolVar(&stdin, "stdin", false, "read webhook query from stdin")
+	cmd.Flags().BoolVar(&stdin, "stdin", !isStdinTerminal, "read webhook query from stdin")
 	cmd.Flags().StringVarP(&url, "url", "u", "", "payload url")
 	cmd.RegisterFlagCompletionFunc("url", cobra.NoFileCompletions)
 	cmd.MarkFlagRequired("url")
@@ -533,10 +652,10 @@ func newMetaUserWebhookListCommand() *cobra.Command {
 		c := createClient("meta", cmd)
 		var cursor *metasrht.Cursor
 
-		pagerify(func(p pager) bool {
+		err := pagerify(func(p pager) error {
 			webhooks, err := metasrht.UserWebhooks(c.Client, ctx, cursor)
 			if err != nil {
-				log.Fatal(err)
+				return err
 			}
 
 			for _, webhook := range webhooks.Results {
@@ -544,8 +663,15 @@ func newMetaUserWebhookListCommand() *cobra.Command {
 			}
 
 			cursor = webhooks.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -585,6 +711,59 @@ func newMetaUserWebhookDeleteCommand() *cobra.Command {
 	return cmd
 }
 
+func newMetaOAuthCommand() *cobra.Command {
+	cmd := &cobra.Command{
+		Use:   "oauth",
+		Short: "List OAuth credentials",
+	}
+	cmd.AddCommand(newMetaOAuthTokensCommand())
+	return cmd
+}
+
+func newMetaOAuthTokensCommand() *cobra.Command {
+	run := func(cmd *cobra.Command, args []string) {
+		ctx := cmd.Context()
+		c := createClient("meta", cmd)
+
+		tokens, err := metasrht.PersonalAccessTokens(c.Client, ctx)
+		if err != nil {
+			log.Fatal(err)
+		}
+
+		tw := tabwriter.NewWriter(os.Stdout, 0, 2, 2, ' ', 0)
+		defer tw.Flush()
+
+		fmt.Fprint(tw, termfmt.Bold.String("Comment\tIssued\tExpires\tGrant\n"))
+		for _, token := range tokens {
+			var s string
+			if token.Comment != nil {
+				s = fmt.Sprintf("%s\t", *token.Comment)
+			} else {
+				s = "\t"
+			}
+
+			issued := humanize.Time(token.Issued.Time)
+			expires := humanize.Time(token.Expires.Time)
+
+			s += fmt.Sprintf("%s\t%s\t", issued, expires)
+
+			if token.Grants != nil {
+				s += *token.Grants
+			}
+
+			fmt.Fprintln(tw, s)
+		}
+	}
+
+	cmd := &cobra.Command{
+		Use:   "tokens",
+		Short: "List personal access tokens",
+		Args:  cobra.ExactArgs(0),
+		Run:   run,
+	}
+	return cmd
+}
+
 func completeSSHKeys(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
 	ctx := cmd.Context()
 	c := createClient("meta", cmd)
