@@ -4,7 +4,7 @@ import (
 	"fmt"
 	"strings"
 
-	"git.sr.ht/~emersion/hut/termfmt"
+	"git.sr.ht/~xenrox/hut/termfmt"
 )
 
 func (visibility Visibility) TermString() string {
@@ -53,3 +53,14 @@ func ParseUserEvents(events []string) ([]WebhookEvent, error) {
 
 	return whEvents, nil
 }
+
+func ParseAccessMode(s string) (AccessMode, error) {
+	switch strings.ToLower(s) {
+	case "ro":
+		return AccessModeRo, nil
+	case "rw":
+		return AccessModeRw, nil
+	default:
+		return "", fmt.Errorf("invalid access mode: %s", s)
+	}
+}
