@@ -11,8 +11,8 @@ import (
 	"os"
 	"strings"
 
-	"git.sr.ht/~emersion/hut/srht/todosrht"
-	"git.sr.ht/~emersion/hut/termfmt"
+	"git.sr.ht/~xenrox/hut/srht/todosrht"
+	"git.sr.ht/~xenrox/hut/termfmt"
 	"github.com/dustin/go-humanize"
 	"github.com/spf13/cobra"
 )
@@ -24,6 +24,7 @@ func newTodoCommand() *cobra.Command {
 	}
 	cmd.AddCommand(newTodoListCommand())
 	cmd.AddCommand(newTodoDeleteCommand())
+	cmd.AddCommand(newTodoUpdateCommand())
 	cmd.AddCommand(newTodoSubscribeCommand())
 	cmd.AddCommand(newTodoUnsubscribeCommand())
 	cmd.AddCommand(newTodoCreateCommand())
@@ -46,21 +47,22 @@ func newTodoListCommand() *cobra.Command {
 		if len(args) > 0 {
 			username = strings.TrimLeft(args[0], ownerPrefixes)
 		}
-		pagerify(func(p pager) bool {
+
+		err := pagerify(func(p pager) error {
 			var trackers *todosrht.TrackerCursor
 			if len(username) > 0 {
 				user, err := todosrht.TrackersByUser(c.Client, ctx, username, cursor)
 				if err != nil {
-					log.Fatal(err)
+					return err
 				} else if user == nil {
-					log.Fatal("no such user")
+					return errors.New("no such user")
 				}
 				trackers = user.Trackers
 			} else {
 				var err error
 				trackers, err = todosrht.Trackers(c.Client, ctx, cursor)
 				if err != nil {
-					log.Fatal(err)
+					return err
 				}
 			}
 
@@ -69,8 +71,15 @@ func newTodoListCommand() *cobra.Command {
 			}
 
 			cursor = trackers.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -95,7 +104,17 @@ func newTodoDeleteCommand() *cobra.Command {
 	var autoConfirm bool
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
-		name, owner, instance := parseResourceName(args[0])
+
+		var name, owner, instance string
+		if len(args) > 0 {
+			name, owner, instance = parseResourceName(args[0])
+		} else {
+			var err error
+			name, owner, instance, err = getTrackerName(ctx, cmd)
+			if err != nil {
+				log.Fatal(err)
+			}
+		}
 		c := createClientWithInstance("todo", cmd, instance)
 		id := getTrackerID(c, ctx, name, owner)
 
@@ -115,9 +134,9 @@ func newTodoDeleteCommand() *cobra.Command {
 	}
 
 	cmd := &cobra.Command{
-		Use:               "delete <tracker>",
+		Use:               "delete [tracker]",
 		Short:             "Delete a tracker",
-		Args:              cobra.ExactArgs(1),
+		Args:              cobra.MaximumNArgs(1),
 		ValidArgsFunction: completeTracker,
 		Run:               run,
 	}
@@ -125,6 +144,105 @@ func newTodoDeleteCommand() *cobra.Command {
 	return cmd
 }
 
+func newTodoUpdateCommand() *cobra.Command {
+	var visibility string
+	var description bool
+	run := func(cmd *cobra.Command, args []string) {
+		ctx := cmd.Context()
+
+		var name, owner, instance string
+		if len(args) > 0 {
+			name, owner, instance = parseResourceName(args[0])
+		} else {
+			var err error
+			name, owner, instance, err = getTrackerName(ctx, cmd)
+			if err != nil {
+				log.Fatal(err)
+			}
+		}
+		c := createClientWithInstance("todo", cmd, instance)
+		id := getTrackerID(c, ctx, name, owner)
+		var input todosrht.TrackerInput
+
+		if visibility != "" {
+			trackerVisibility, err := todosrht.ParseVisibility(visibility)
+			if err != nil {
+				log.Fatal(err)
+			}
+			input.Visibility = &trackerVisibility
+		}
+
+		if description {
+			if !isStdinTerminal {
+				b, err := io.ReadAll(os.Stdin)
+				if err != nil {
+					log.Fatalf("failed to read description: %v", err)
+				}
+				description := string(b)
+				input.Description = &description
+			} else {
+				var (
+					err      error
+					user     *todosrht.User
+					username string
+				)
+
+				if owner != "" {
+					username = strings.TrimLeft(owner, ownerPrefixes)
+					user, err = todosrht.TrackerDescriptionByUser(c.Client, ctx, username, name)
+				} else {
+					user, err = todosrht.TrackerDescription(c.Client, ctx, name)
+				}
+
+				if err != nil {
+					log.Fatalf("failed to fetch description: %v", err)
+				} else if user == nil {
+					log.Fatalf("no such user %q", username)
+				} else if user.Tracker == nil {
+					log.Fatalf("no such tracker %q", name)
+				}
+
+				var prefill string
+				if user.Tracker.Description != nil {
+					prefill = *user.Tracker.Description
+				}
+
+				text, err := getInputWithEditor("hut_description*.md", prefill)
+				if err != nil {
+					log.Fatalf("failed to read description: %v", err)
+				}
+
+				if strings.TrimSpace(text) == "" {
+					_, err := todosrht.ClearDescription(c.Client, ctx, id)
+					if err != nil {
+						log.Fatalf("failed to clear description: %v", err)
+					}
+				} else {
+					input.Description = &text
+				}
+			}
+		}
+
+		tracker, err := todosrht.UpdateTracker(c.Client, ctx, id, input)
+		if err != nil {
+			log.Fatal(err)
+		}
+		log.Printf("Updated tracker %q\n", tracker.Name)
+	}
+
+	cmd := &cobra.Command{
+		Use:               "update [tracker]",
+		Short:             "Update a tracker",
+		Args:              cobra.MaximumNArgs(1),
+		ValidArgsFunction: completeTracker,
+		Run:               run,
+	}
+	cmd.Flags().StringVarP(&visibility, "visibility", "v", "", "tracker visibility")
+	cmd.RegisterFlagCompletionFunc("visibility", completeVisibility)
+	cmd.Flags().BoolVar(&description, "description", false, "edit description")
+	return cmd
+}
+
 func newTodoSubscribeCommand() *cobra.Command {
 	run := func(cmd *cobra.Command, args []string) {
 		ctx := cmd.Context()
@@ -254,7 +372,7 @@ func newTodoCreateCommand() *cobra.Command {
 	}
 	cmd.Flags().StringVarP(&visibility, "visibility", "v", "public", "tracker visibility")
 	cmd.RegisterFlagCompletionFunc("visibility", completeVisibility)
-	cmd.Flags().BoolVar(&stdin, "stdin", false, "read tracker from stdin")
+	cmd.Flags().BoolVar(&stdin, "stdin", !isStdinTerminal, "read tracker from stdin")
 	return cmd
 }
 
@@ -306,7 +424,7 @@ func newTodoTicketListCommand() *cobra.Command {
 			username = strings.TrimLeft(owner, ownerPrefixes)
 		}
 
-		pagerify(func(p pager) bool {
+		err = pagerify(func(p pager) error {
 			if username != "" {
 				user, err = todosrht.TicketsByUser(c.Client, ctx, username, name, cursor)
 			} else {
@@ -314,11 +432,11 @@ func newTodoTicketListCommand() *cobra.Command {
 			}
 
 			if err != nil {
-				log.Fatal(err)
+				return err
 			} else if user == nil {
-				log.Fatalf("no such user %q", username)
+				return fmt.Errorf("no such user %q", username)
 			} else if user.Tracker == nil {
-				log.Fatalf("no such tracker %q", name)
+				return fmt.Errorf("no such tracker %q", name)
 			}
 
 			for _, ticket := range user.Tracker.Tickets.Results {
@@ -330,8 +448,15 @@ func newTodoTicketListCommand() *cobra.Command {
 			}
 
 			cursor = user.Tracker.Tickets.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 
 	}
 
@@ -449,7 +574,7 @@ func newTodoTicketCommentCommand() *cobra.Command {
 		ValidArgsFunction: completeTicketID,
 		Run:               run,
 	}
-	cmd.Flags().BoolVar(&stdin, "stdin", false, "read comment from stdin")
+	cmd.Flags().BoolVar(&stdin, "stdin", !isStdinTerminal, "read comment from stdin")
 	cmd.Flags().StringVarP(&status, "status", "s", "", "ticket status")
 	cmd.RegisterFlagCompletionFunc("status", completeTicketStatus)
 	cmd.Flags().StringVarP(&resolution, "resolution", "r", "", "ticket resolution")
@@ -501,8 +626,6 @@ func newTodoTicketStatusCommand() *cobra.Command {
 		event, err := todosrht.UpdateTicketStatus(c.Client, ctx, trackerID, ticketID, input)
 		if err != nil {
 			log.Fatal(err)
-		} else if event == nil {
-			log.Fatalf("failed to update status of ticket with ID %d", ticketID)
 		}
 
 		log.Printf("Updated status of %s\n", event.Ticket.Subject)
@@ -741,7 +864,8 @@ func newTodoTicketShowCommand() *cobra.Command {
 		}
 
 		ticket := user.Tracker.Ticket
-		fmt.Printf("%s\n\n", termfmt.Bold.String(ticket.Subject))
+		fmt.Printf("%s\n", termfmt.Bold.String(ticket.Subject))
+		fmt.Printf("%s/%s/%s/%d\n\n", c.BaseURL, owner, name, ticketID)
 
 		fmt.Printf("Status: %s\n", termfmt.Green.Sprintf("%s %s", ticket.Status, ticket.Resolution))
 		fmt.Printf("Submitter: %s\n", ticket.Submitter.CanonicalName)
@@ -860,7 +984,7 @@ func newTodoTicketWebhookCreateCommand() *cobra.Command {
 	cmd.Flags().StringSliceVarP(&events, "events", "e", nil, "webhook events")
 	cmd.RegisterFlagCompletionFunc("events", completeTicketWebhookEvents)
 	cmd.MarkFlagRequired("events")
-	cmd.Flags().BoolVar(&stdin, "stdin", false, "read webhook query from stdin")
+	cmd.Flags().BoolVar(&stdin, "stdin", !isStdinTerminal, "read webhook query from stdin")
 	cmd.Flags().StringVarP(&url, "url", "u", "", "payload url")
 	cmd.RegisterFlagCompletionFunc("url", cobra.NoFileCompletions)
 	cmd.MarkFlagRequired("url")
@@ -886,7 +1010,7 @@ func newTodoTicketWebhookListCommand() *cobra.Command {
 			username = strings.TrimLeft(owner, ownerPrefixes)
 		}
 
-		pagerify(func(p pager) bool {
+		err = pagerify(func(p pager) error {
 			if username != "" {
 				user, err = todosrht.TicketWebhooksByUser(c.Client, ctx, username, name, ticketID, cursor)
 			} else {
@@ -894,11 +1018,11 @@ func newTodoTicketWebhookListCommand() *cobra.Command {
 			}
 
 			if err != nil {
-				log.Fatal(err)
+				return err
 			} else if user == nil {
-				log.Fatalf("no such user %q", username)
+				return fmt.Errorf("no such user %q", username)
 			} else if user.Tracker == nil {
-				log.Fatalf("no such tracker %q", name)
+				return fmt.Errorf("no such tracker %q", name)
 			}
 
 			for _, webhook := range user.Tracker.Ticket.Webhooks.Results {
@@ -906,8 +1030,15 @@ func newTodoTicketWebhookListCommand() *cobra.Command {
 			}
 
 			cursor = user.Tracker.Ticket.Webhooks.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -956,6 +1087,7 @@ func newTodoLabelCommand() *cobra.Command {
 	cmd.AddCommand(newTodoLabelListCommand())
 	cmd.AddCommand(newTodoLabelDeleteCommand())
 	cmd.AddCommand(newTodoLabelCreateCommand())
+	cmd.AddCommand(newTodoLabelUpdateCommand())
 	return cmd
 }
 
@@ -977,7 +1109,7 @@ func newTodoLabelListCommand() *cobra.Command {
 			username = strings.TrimLeft(owner, ownerPrefixes)
 		}
 
-		pagerify(func(p pager) bool {
+		err = pagerify(func(p pager) error {
 			if username != "" {
 				user, err = todosrht.LabelsByUser(c.Client, ctx, username, name, cursor)
 			} else {
@@ -985,11 +1117,11 @@ func newTodoLabelListCommand() *cobra.Command {
 			}
 
 			if err != nil {
-				log.Fatal(err)
+				return err
 			} else if user == nil {
-				log.Fatalf("no such user %q", username)
+				return fmt.Errorf("no such user %q", username)
 			} else if user.Tracker == nil {
-				log.Fatalf("no such tracker %q", name)
+				return fmt.Errorf("no such tracker %q", name)
 			}
 
 			for _, label := range user.Tracker.Labels.Results {
@@ -997,8 +1129,15 @@ func newTodoLabelListCommand() *cobra.Command {
 			}
 
 			cursor = user.Tracker.Labels.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -1042,6 +1181,56 @@ func newTodoLabelDeleteCommand() *cobra.Command {
 	return cmd
 }
 
+func newTodoLabelUpdateCommand() *cobra.Command {
+	var bg, fg, name string
+	run := func(cmd *cobra.Command, args []string) {
+		ctx := cmd.Context()
+		trackerName, owner, instance, err := getTrackerName(ctx, cmd)
+		if err != nil {
+			log.Fatal(err)
+		}
+
+		c := createClientWithInstance("todo", cmd, instance)
+		id, err := getLabelID(c, ctx, trackerName, args[0], owner)
+		if err != nil {
+			log.Fatalf("failed to get label ID: %v", err)
+		}
+
+		var input todosrht.UpdateLabelInput
+		if fg != "" {
+			input.ForegroundColor = &fg
+		}
+		if bg != "" {
+			input.ForegroundColor = &bg
+		}
+		if name != "" {
+			input.Name = &name
+		}
+
+		label, err := todosrht.UpdateLabel(c.Client, ctx, id, input)
+		if err != nil {
+			log.Fatal(err)
+		}
+
+		log.Printf("Updated label %s\n", label.Name)
+	}
+
+	cmd := &cobra.Command{
+		Use:               "update <name>",
+		Short:             "Update a label",
+		Args:              cobra.ExactArgs(1),
+		ValidArgsFunction: completeLabel,
+		Run:               run,
+	}
+	cmd.Flags().StringVarP(&fg, "foreground", "f", "", "foreground color")
+	cmd.RegisterFlagCompletionFunc("foreground", completeLabelColor)
+	cmd.Flags().StringVarP(&bg, "background", "b", "", "background color")
+	cmd.RegisterFlagCompletionFunc("background", completeLabelColor)
+	cmd.Flags().StringVarP(&name, "name", "n", "", "label name")
+	cmd.RegisterFlagCompletionFunc("name", cobra.NoFileCompletions)
+	return cmd
+}
+
 func newTodoLabelCreateCommand() *cobra.Command {
 	var fg, bg string
 	run := func(cmd *cobra.Command, args []string) {
@@ -1118,7 +1307,7 @@ func newTodoACLListCommand() *cobra.Command {
 			username = strings.TrimLeft(owner, ownerPrefixes)
 		}
 
-		pagerify(func(p pager) bool {
+		err = pagerify(func(p pager) error {
 			if username != "" {
 				user, err = todosrht.AclByUser(c.Client, ctx, username, name, cursor)
 			} else {
@@ -1126,11 +1315,11 @@ func newTodoACLListCommand() *cobra.Command {
 			}
 
 			if err != nil {
-				log.Fatal(err)
+				return err
 			} else if user == nil {
-				log.Fatalf("no such user %q", username)
+				return fmt.Errorf("no such user %q", username)
 			} else if user.Tracker == nil {
-				log.Fatalf("no such tracker %q", name)
+				return fmt.Errorf("no such tracker %q", name)
 			}
 
 			if cursor == nil {
@@ -1148,8 +1337,15 @@ func newTodoACLListCommand() *cobra.Command {
 			}
 
 			cursor = user.Tracker.Acls.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -1262,7 +1458,7 @@ func newTodoWebhookCreateCommand() *cobra.Command {
 	cmd.Flags().StringSliceVarP(&events, "events", "e", nil, "webhook events")
 	cmd.RegisterFlagCompletionFunc("events", completeTrackerWebhookEvents)
 	cmd.MarkFlagRequired("events")
-	cmd.Flags().BoolVar(&stdin, "stdin", false, "read webhook query from stdin")
+	cmd.Flags().BoolVar(&stdin, "stdin", !isStdinTerminal, "read webhook query from stdin")
 	cmd.Flags().StringVarP(&url, "url", "u", "", "payload url")
 	cmd.RegisterFlagCompletionFunc("url", cobra.NoFileCompletions)
 	cmd.MarkFlagRequired("url")
@@ -1296,7 +1492,7 @@ func newTodoWebhookListCommand() *cobra.Command {
 			username = strings.TrimLeft(owner, ownerPrefixes)
 		}
 
-		pagerify(func(p pager) bool {
+		err = pagerify(func(p pager) error {
 			if username != "" {
 				user, err = todosrht.TrackerWebhooksByUser(c.Client, ctx, username, name, cursor)
 			} else {
@@ -1304,11 +1500,11 @@ func newTodoWebhookListCommand() *cobra.Command {
 			}
 
 			if err != nil {
-				log.Fatal(err)
+				return err
 			} else if user == nil {
-				log.Fatalf("no such user %q", username)
+				return fmt.Errorf("no such user %q", username)
 			} else if user.Tracker == nil {
-				log.Fatalf("no such tracker %q", name)
+				return fmt.Errorf("no such tracker %q", name)
 			}
 
 			for _, webhook := range user.Tracker.Webhooks.Results {
@@ -1316,8 +1512,15 @@ func newTodoWebhookListCommand() *cobra.Command {
 			}
 
 			cursor = user.Tracker.Webhooks.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 
 	}
 
@@ -1406,7 +1609,7 @@ func newTodoUserWebhookCreateCommand() *cobra.Command {
 	cmd.Flags().StringSliceVarP(&events, "events", "e", nil, "webhook events")
 	cmd.RegisterFlagCompletionFunc("events", completeTodoUserWebhookEvents)
 	cmd.MarkFlagRequired("events")
-	cmd.Flags().BoolVar(&stdin, "stdin", false, "read webhook query from stdin")
+	cmd.Flags().BoolVar(&stdin, "stdin", !isStdinTerminal, "read webhook query from stdin")
 	cmd.Flags().StringVarP(&url, "url", "u", "", "payload url")
 	cmd.RegisterFlagCompletionFunc("url", cobra.NoFileCompletions)
 	cmd.MarkFlagRequired("url")
@@ -1419,10 +1622,10 @@ func newTodoUserWebhookListCommand() *cobra.Command {
 		c := createClient("todo", cmd)
 		var cursor *todosrht.Cursor
 
-		pagerify(func(p pager) bool {
+		err := pagerify(func(p pager) error {
 			webhooks, err := todosrht.UserWebhooks(c.Client, ctx, cursor)
 			if err != nil {
-				log.Fatal(err)
+				return err
 			}
 
 			for _, webhook := range webhooks.Results {
@@ -1430,8 +1633,15 @@ func newTodoUserWebhookListCommand() *cobra.Command {
 			}
 
 			cursor = webhooks.Cursor
-			return cursor == nil
+			if cursor == nil {
+				return pagerDone
+			}
+
+			return nil
 		})
+		if err != nil {
+			log.Fatal(err)
+		}
 	}
 
 	cmd := &cobra.Command{
@@ -1550,7 +1760,7 @@ func newTodoTicketCreateCommand() *cobra.Command {
 		Args:  cobra.ExactArgs(0),
 		Run:   run,
 	}
-	cmd.Flags().BoolVar(&stdin, "stdin", false, "read ticket from stdin")
+	cmd.Flags().BoolVar(&stdin, "stdin", !isStdinTerminal, "read ticket from stdin")
 	return cmd
 }
 
@@ -1614,8 +1824,6 @@ func newTodoTicketEditCommand() *cobra.Command {
 		ticket, err = todosrht.UpdateTicket(c.Client, ctx, tracker.Id, ticket.Id, input)
 		if err != nil {
 			log.Fatal(err)
-		} else if ticket == nil {
-			log.Fatal("failed to edit ticket")
 		}
 
 		log.Printf("Updated ticket %v\n", termfmt.DarkYellow.Sprintf("#%v", ticket.Id))
@@ -1666,8 +1874,7 @@ func newTodoTicketLabelCommand() *cobra.Command {
 	}
 	cmd.Flags().StringVarP(&labelName, "label", "l", "", "label name")
 	cmd.MarkFlagRequired("label")
-	// TODO: complete unassigned labels
-	cmd.RegisterFlagCompletionFunc("label", cobra.NoFileCompletions)
+	cmd.RegisterFlagCompletionFunc("label", completeTicketLabel)
 	return cmd
 }
 
@@ -1706,8 +1913,7 @@ func newTodoTicketUnlabelCommand() *cobra.Command {
 	}
 	cmd.Flags().StringVarP(&labelName, "label", "l", "", "label name")
 	cmd.MarkFlagRequired("label")
-	// TODO: complete assigned labels
-	cmd.RegisterFlagCompletionFunc("label", cobra.NoFileCompletions)
+	cmd.RegisterFlagCompletionFunc("label", completeTicketUnlabel)
 	return cmd
 }
 
@@ -1744,6 +1950,15 @@ func getTrackerName(ctx context.Context, cmd *cobra.Command) (name, owner, insta
 		return name, owner, instance, nil
 	}
 
+	cfg, err := loadProjectConfig()
+	if err != nil {
+		return "", "", "", err
+	}
+	if cfg != nil && cfg.Tracker != "" {
+		name, owner, instance = parseResourceName(cfg.Tracker)
+		return name, owner, instance, nil
+	}
+
 	// TODO: Use hub.sr.ht API to determine trackers
 	name, owner, instance, err = guessGitRepoName(ctx, cmd)
 	if err != nil {
@@ -2120,3 +2335,80 @@ func completeLabel(cmd *cobra.Command, args []string, toComplete string) ([]stri
 
 	return labelList, cobra.ShellCompDirectiveNoFileComp
 }
+
+func completeTicketLabel(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	// display all labels if no ticket is specified
+	if len(args) == 0 {
+		return completeLabel(cmd, args, toComplete)
+	}
+
+	ctx := cmd.Context()
+	ticketID, name, owner, instance, err := parseTicketResource(ctx, cmd, args[0])
+	if err != nil {
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+
+	c := createClientWithInstance("todo", cmd, instance)
+	var (
+		user *todosrht.User
+	)
+
+	if owner != "" {
+		username := strings.TrimLeft(owner, ownerPrefixes)
+		user, err = todosrht.CompleteTicketLabelByUser(c.Client, ctx, username, name, ticketID)
+	} else {
+		user, err = todosrht.CompleteTicketLabel(c.Client, ctx, name, ticketID)
+	}
+	if err != nil || user == nil || user.Tracker == nil {
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+
+	var ticketLabels []string
+	for _, label := range user.Tracker.Ticket.Labels {
+		ticketLabels = append(ticketLabels, label.Name)
+	}
+
+	var labelList []string
+	for _, label := range user.Tracker.Labels.Results {
+		if !sliceContains(ticketLabels, label.Name) {
+			labelList = append(labelList, label.Name)
+		}
+	}
+
+	return labelList, cobra.ShellCompDirectiveNoFileComp
+}
+
+func completeTicketUnlabel(cmd *cobra.Command, args []string, toComplete string) ([]string, cobra.ShellCompDirective) {
+	// display all labels if no ticket is specified
+	if len(args) == 0 {
+		return completeLabel(cmd, args, toComplete)
+	}
+
+	ctx := cmd.Context()
+	ticketID, name, owner, instance, err := parseTicketResource(ctx, cmd, args[0])
+	if err != nil {
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+
+	c := createClientWithInstance("todo", cmd, instance)
+	var (
+		user *todosrht.User
+	)
+
+	if owner != "" {
+		username := strings.TrimLeft(owner, ownerPrefixes)
+		user, err = todosrht.CompleteTicketUnlabelByUser(c.Client, ctx, username, name, ticketID)
+	} else {
+		user, err = todosrht.CompleteTicketUnlabel(c.Client, ctx, name, ticketID)
+	}
+	if err != nil || user == nil || user.Tracker == nil {
+		return nil, cobra.ShellCompDirectiveNoFileComp
+	}
+
+	var labelList []string
+	for _, label := range user.Tracker.Ticket.Labels {
+		labelList = append(labelList, label.Name)
+	}
+
+	return labelList, cobra.ShellCompDirectiveNoFileComp
+}
